﻿/*	AVR-HID/IO
	『昼夜逆転』工作室 http://jsdiy.web.fc2.com

【開発環境】
WindowsXP Pro. 32bit SP3 (SP1[OEM/CD-ROM] -> SP3[online update])
VisualStudio2008 Exp.Edition - C#
※本プログラムはWindowsに組み込まれているDLLを使用します。hid.dll, setupapi.dll, kernel32.dll

【更新履歴】
2012/04/02	ver 1.00	初版
*/

using System;
using UsbHid;

namespace UsbHid.IOCtrl
{
	enum Cmd
	{
		//register operation
		CMD_GET,
		CMD_SET,
		CMD_AND,
		CMD_OR,
		CMD_XOR,

		//EEPROM operation
		CMD_READ_BYTE,
		CMD_WRITE_BYTE,
		CMD_READ_WORD,
		CMD_WRITE_WORD
	};

	enum CmdIdx
	{
		//cmdBuf[0:Cmd][1:reg][2:value-Lo][3:value-Hi][4:adrs-Lo][5:adrs-Hi]
		ICmd = 0,
		IReg,
		IValL, IVal8 = IValL,
		IValH,
		IAdrsL,
		IAdrsH
	};
	
	class HidIOCtrl
	{
		public bool IsSuccess { get; protected set; }
		protected readonly HidDevice hid;
		protected byte[] cmdBuf = new byte[6];
		
		public HidIOCtrl(HidDevice hiddev)
		{
			hid = hiddev;
			IsSuccess = false;
		}

		//レジスタを操作する
		protected void MakeCmdBufReg(Cmd command, SFR register, int byteValue)
		{
			for (int i = 0; i < cmdBuf.Length; i++) cmdBuf[i] = 0x00;
			cmdBuf[(int)CmdIdx.ICmd] = (byte)command;
			cmdBuf[(int)CmdIdx.IReg] = (byte)register;
			cmdBuf[(int)CmdIdx.IVal8] = (byte)(byteValue & 0xFF);
		}

		public int Get(SFR register)
		{
			MakeCmdBufReg(Cmd.CMD_GET, register, 0);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			if (!IsSuccess) return 0;
			byte[] data = hid.GetFeatureReport(cmdBuf.Length);
			return (int)data[(int)CmdIdx.IVal8];
		}

		public bool Set(SFR register, int byteValue)
		{
			MakeCmdBufReg(Cmd.CMD_SET, register, byteValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		public bool AndSet(SFR register, int byteValue)
		{
			MakeCmdBufReg(Cmd.CMD_AND, register, byteValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		public bool OrSet(SFR register, int byteValue)
		{
			MakeCmdBufReg(Cmd.CMD_OR, register, byteValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		public bool XorSet(SFR register, int byteValue)
		{
			MakeCmdBufReg(Cmd.CMD_XOR, register, byteValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		//EEPROMを操作する
		protected void MakeCmdBufEEPRom(Cmd command, int adrs, int nValue)
		{
			for (int i = 0; i < cmdBuf.Length; i++) cmdBuf[i] = 0x00;
			cmdBuf[(int)CmdIdx.ICmd] = (byte)command;
			cmdBuf[(int)CmdIdx.IAdrsH] = (byte)((adrs >> 8) & 0xFF);
			cmdBuf[(int)CmdIdx.IAdrsL] = (byte)(adrs & 0xFF);
			cmdBuf[(int)CmdIdx.IValH] = (byte)((nValue >> 8) & 0xFF);
			cmdBuf[(int)CmdIdx.IValL] = (byte)(nValue & 0xFF);
		}

		public byte EEPRomReadByte(int adrs)
		{
			MakeCmdBufEEPRom(Cmd.CMD_READ_BYTE, adrs, 0);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			if (!IsSuccess) return 0;
			byte[] data = hid.GetFeatureReport(cmdBuf.Length);
			return data[(int)CmdIdx.IVal8];
		}

		public bool EEPRomWriteByte(int adrs, int byteValue)
		{
			MakeCmdBufEEPRom(Cmd.CMD_WRITE_BYTE, adrs, byteValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		public int EEPRomReadWord(int adrs)
		{
			MakeCmdBufEEPRom(Cmd.CMD_READ_WORD, adrs, 0);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			if (!IsSuccess) return 0;
			byte[] data = hid.GetFeatureReport(cmdBuf.Length);
			int valueH = data[(int)CmdIdx.IValH];
			int valueL = data[(int)CmdIdx.IValL];
			int nValue = (valueH << 8) | valueL;
			return nValue;
		}

		public bool EEPRomWriteWord(int adrs, int nValue)
		{
			MakeCmdBufEEPRom(Cmd.CMD_WRITE_WORD, adrs, nValue);
			IsSuccess = hid.SetFeatureReport(cmdBuf);
			return IsSuccess;
		}

		//割り込みの受信データを、ベクター番号を示すビットフラグに変換する
		//引数: インタラプト転送(IN)のコールバック関数で受け取った引数
		/*
		・data[0]はレポートID。実データは[1]以降。
		・データ部は32bitのビットフラグとして見る。MSB[4][3][2][1]LSB
		・ビットnがベクター番号nを表す（MSB=bit:31, LSB=bit:0）。
			※ベクター番号0というものは存在しない。
			※ベクター番号1はリセットベクターであり、ビットフラグが立つことはない。
			※INT0,INT1割り込みはV-USBで使用しており、対応するベクター番号のビットフラグが立つことはない。
		*/
		public static UInt32 ToBitFlag(byte[] data)
		{
			if (data == null || data.Length < 1 + 4) return 0;

			UInt32 vectorBits = 0;
			for (int i = 0; i < 4; i++)
				vectorBits |= ((UInt32)data[1 + i] << (8 * i));
			return vectorBits;
		}
		
	}	//end of class
	
}	//end of namespace
